package ROTool;
use strict;
use IO::Socket::INET;

my $VERSION = "ROTool.pm_v0.2.2";
sub version{
    return $VERSION;
}

my $timeout = 10;
sub timeout(){
    if(scalar(@_) > 0){
	$timeout = $_[0];
    }
    return $timeout;
}

my $keeptime = 10800;
sub keeptime{
    if(scalar(@_) > 0){
	$keeptime = $_[0];
    }
    return $keeptime;
}

my %UA;
my %clients;
my %sock_to_client;

sub UA_regist{
    my($id,$client) = @_; 
    if(not exists $UA{$id}){
	$UA{$id} = $client;
	$client->{UA_registed} = 1;
	&ROTool::printlog(2," regist UA[id = $id]\n");
	return 1;
    }
   if($UA{$id} == $client){
	return 1;
    }else{
	if($UA{$id}->{"Message_lasting"}){
	    my $oldclient = $UA{$id};
	    $UA{$id} = $client;
	    $client->{UA_registed} = 1;
	    undef $oldclient->{UA_registed};
	    $oldclient->shutdown();
	    &ROTool::printlog(2," override regist UA[id = $id]\n");
	    return 1;
	}else{
	    &ROTool::printlog(2," reject UA[id = $id]\n");
	    return 0;
	}
    }
}

sub UA_unregist{
    my $id = shift;
    if(ref($id)){
	return if($id->{timer}->{shutdown} > 0);
	$id = $id->{"param"}->{"id"};
	foreach my $c (values %clients){
	    return if($c->{"param"}->{"id"} eq $id &&
		      $c->{"timer"}->{"shutdown"} > 0);
	}
    }
    &ROTool::printlog(2," unregist UA[id = $id]\n");
    delete $UA{$id};
}

sub client_regist{
    my $client = shift;
    $clients{$client->oid()} = $client;
    $sock_to_client{$client->{"sock"}} = $client;
    &ROTool::printlog(2," regist client[oid = ".$client->oid()."]\n");
}
sub client_unregist{
    my $client = shift;
    &ROTool::printlog(2," unregist client[oid = ".$client->oid()."]\n");
    delete $clients{$client->oid()};
}
sub sock_unregist{
    my $client = shift;
    delete $sock_to_client{$client->{"sock"}};
    
}

sub UA_list{
    return values %UA;
}
sub clients_all{
    return values %clients;
}
sub sock_to_client{
    my $sock = shift;
    return $sock_to_client{$sock};
}

my @typechecker;
sub typechecker{
    return @typechecker;
}
sub typechecker_regist{
    my $typechecker = shift;
    push(@typechecker,$typechecker);
}

#Logging
my($loglevel) = 1;
sub loglevel{
    my $ret = $loglevel;
    if(scalar(@_) > 0){
	$loglevel = shift;
    }
    return $ret;
}

sub printlog{
    my($lv) = shift;
    if($lv <= $loglevel && scalar(@_) > 0){
	&main::printlog(@_);
    }
}

# methods ROTool object.
sub is_client{
    return 0;
}

sub is_server{
    return 0;
}

my $oid_serial = 1;
sub new{
    my $class = shift;
    my $sock  = shift;
    my $opt   = shift;
    my $blockIO = 0;
    my $autoflash = 1;

    if(not defined $sock->blocking(0)){
	&ROTool::printlog(2," sock non blocking fail : $!\n");
	$blockIO = 1;
    }

    if(not defined $sock->autoflush(1)){
	&ROTool::printlog(2," sock autoflush fail : $!\n");
	$autoflash = 0;
    }

    my $self = {};
    $self->{"buf"} = "";
    $self->{"sock"} = $sock;
    $self->{"timer"}->{"shutdown"} = $timeout;
    $self->{"oid"} = $oid_serial++;
    $self->{"conf"} = {	"blockIO"   => $blockIO ,
			"autoflash" => $autoflash };
    $self->{"opt"} = $opt;
    &ROTool::printlog(1," accept new ROTool [$self->{oid}]\n");
    return bless $self,$class;
}

sub oid{
    my $self = shift;
    return $self->{"oid"};
}

sub recv{
    my $self = shift;

    my $sock = $self->{"sock"};
    my $len = $sock->sysread($self->{"buf"},8192,length($self->{"buf"}));
    foreach(&ROTool::typechecker()){
	&ROTool::printlog(2,"  check by $_\n");
	if(&$_($self)){
	    $self->{"flag"}->{"noread"} = 1;
	    return $self->recv;
	}
    }
    return $len;
}

sub send{
    my $self = shift;
    my(%clients) = @_;

    return;
}

sub shutdown{
    my $self = shift;
    my($selecter) = @_;

    my $sock = $self->{"sock"};
    &ROTool::sock_unregist($self);
    &ROTool::client_unregist($self);
    $selecter->remove($sock);
    $sock->shutdown(2);
    &ROTool::printlog(1," shutdown [$self->{oid}]\n");
}

sub is_keep_message{
    return 0;
}

sub keep_message{
}

sub checktimer{
    my $self = shift;
    my($selecter,$sock_to_client,$clients) = @_;

    my @result;
    $self->{timer}->{shutdown}--;
    if($self->{timer}->{shutdown} <= 0){
	&ROTool::printlog(1," ROTools shutdowns [".$self->oid()."](UnDefined client)\n");
	$self->shutdown($selecter,$sock_to_client,$clients);
    }
}

sub set_schedule{
    my $self    = shift;
    my $command = shift;
    if($command eq 'send'){
	$self->{"timer"}->{"send"} = 1;
    }
}

1;
